__version__ = '0.1.6'
__gh = 'https://github.com/btskinner/'
__nm = 'grm'
__rp = '{}{}'.format(__gh, __nm)
__dl = '{}{}/archive/{}-v{}.tar.gz'.format(__gh, __nm, __nm, __version__)
__au = 'Benjamin Skinner'
__em = 'b.skinner@vanderbilt.edu'
